EntryAction

Enum EntryAction 

pub enum EntryAction<T> {
    Retain(T),
    Remove,
    ReplaceWithGuard,
}
What to do with an existing entry after inspection/mutation.

Used with Cache::entry and Cache::entry_async.

Variants



Retain(T)

Retain the entry in the cache. The value may have been mutated in place before returning this variant.

Returns EntryResult::Retained(T).



Remove

Remove the entry from the cache.

Returns EntryResult::Removed(Key, Val).



ReplaceWithGuard

Remove the entry and get a PlaceholderGuard for re-insertion. This is useful for “validate-or-recompute” patterns.

Returns EntryResult::Replaced(PlaceholderGuard, Val).