pub struct LockContention;Expand description
Error returned by non-blocking cache operations that do not consume their inputs when the relevant shard lock could not be acquired immediately.
This is used by borrowed-key/read-path operations. Non-blocking operations
that consume owned inputs (e.g. try_insert) instead return those inputs
on contention so the caller can retry or discard without losing data.